@@ -31,7 +31,9 @@
 int verbose = 0;
 int savefiles = 0;
 int saveRTF = 0;
+int saveHTML = 0;
 int saveintermediate = 0;
+int attachmentSize = 50;
 char *filepath = NULL;
 int ProcessTNEF(TNEFStruct TNEF);
 void SaveVCalendar(TNEFStruct TNEF, int isMtgReq);
@@ -50,7 +52,9 @@ void PrintHelp(void) {
   printf("          Multiple -v's increase the level of output\n");
   printf("   -/+f - Enables/Disables saving of attachments\n");
   printf("   -/+F - Enables/Disables saving of the message body as RTF\n");
+  printf("   -/+H - Enables/Disables saving of the message body as HTML\n");
   printf("   -/+a - Enables/Disables saving of intermediate files\n");
+  printf("   -s <n> - Override the default Attachment Size (%i MB)\n", attachmentSize);
   printf("   -h   - Displays this help message\n");
   printf("\n");
   printf("Example:\n");
@@ -94,10 +98,17 @@ int main(int argc, char **argv) {
           filepath = argv[i + 1];
           i++;
           break;
+        case 's': 
+          attachmentSize = atoi(argv[i+1]);
+          i++;
+          break;
         case 'F': saveRTF = 1;
           break;
+        case 'H': saveHTML = 1;
+          break;
         default:
           printf("Unknown option '%s'\n", argv[i]);
+	  PrintHelp();	      
       }
       continue;
 
@@ -113,8 +124,11 @@ int main(int argc, char **argv) {
           break;
         case 'F': saveRTF = 0;
           break;
+        case 'H': saveHTML = 0;
+          break;
         default:
           printf("Unknown option '%s'\n", argv[i]);
+	  PrintHelp();	      
       }
       continue;
 
@@ -122,6 +136,7 @@ int main(int argc, char **argv) {
 
     TNEFInitialize(&TNEF);
     TNEF.Debug = verbose;
+    TNEF.attachmentSize = attachmentSize;
     if (TNEFParseFile(argv[i], &TNEF) == -1) {
       fprintf(stderr, "ERROR processing file\n");
       ++errors;
@@ -208,6 +223,35 @@ int ProcessTNEF(TNEFStruct TNEF) {
         }
       }
     }
+
+    if ((saveHTML == 1) && (TNEF.subject.size > 0)) {
+      // filename seems to be the actual data rather than the filename
+      if ((filename = MAPIFindProperty(&(TNEF.MapiProperties),
+              PROP_TAG(PT_BINARY, PR_BODY_HTML)))
+          != MAPI_UNDEFINED) {
+        int size;
+        variableLength buf;
+        snprintf(ifilename, MAX_FILENAME_SIZE, "%s.html", TNEF.subject.data);
+        SanitizeFilename(ifilename);
+        if (filepath != NULL) {
+          char tmp[MAX_FILENAME_SIZE+1];
+          snprintf(tmp, MAX_FILENAME_SIZE, "%s/%s", filepath, ifilename);
+          strcpy(ifilename, tmp);
+        }
+
+        if ((fptr = fopen(ifilename, "wb")) == NULL) {
+          fprintf(stderr, "ERROR: Error writing %s to disk!\n", ifilename);
+          ++failures;
+        } else {
+          fwrite(filename->data,
+              sizeof(BYTE),
+              filename->size,
+              fptr);
+          fclose(fptr);
+          printf("%s\n", ifilename);
+        }
+      }
+    }
   }
 
   if (strcmp(TNEF.messageClass, "IPM.StickyNote") == 0) {
@@ -286,6 +330,7 @@ int ProcessTNEF(TNEFStruct TNEF) {
         if (TNEFCheckForSignature(signature) == 0) {
           // Has a TNEF signature, so process it.
           TNEFInitialize(&emb_tnef);
+          emb_tnef.attachmentSize = attachmentSize;
           emb_tnef.Debug = TNEF.Debug;
           if (TNEFParseMemory(filedata->data + 16,
                               filedata->size - 16, &emb_tnef) != -1) {
@@ -301,6 +346,7 @@ int ProcessTNEF(TNEFStruct TNEF) {
         if (TNEFCheckForSignature(signature) == 0) {
           // Has a TNEF signature, so process it.
           TNEFInitialize(&emb_tnef);
+          emb_tnef.attachmentSize = attachmentSize;
           emb_tnef.Debug = TNEF.Debug;
           if (TNEFParseMemory(filedata->data,
                               filedata->size, &emb_tnef) != -1) {
@@ -332,7 +378,7 @@ int ProcessTNEF(TNEFStruct TNEF) {
           filename->data = (BYTE *)malloc(20);
           snprintf((char*)filename->data, 19, "file_%03i.dat", count);
         }
-        snprintf(ifilename, MAX_FILENAME_SIZE, "%s", filename->data);
+        snprintf(ifilename, MAX_FILENAME_SIZE, "%.*s", filename->size, filename->data);
         for(i = 0; i < strlen(ifilename); i++) 
           if (ifilename[i] == ' ') 
             ifilename[i] = '_';
